package it.unibz.inf.completeness.experiment;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

public class Measurement {

	public double reasoningTime;
	public double hashmapLookupsTime;
	public double tcApplicationTime;
	public int queryLength;
	public int numOfRelevantStatements;
	public int numOfConstantsOfQuery;
	
	public Measurement(double reasoningTime, int queryLength, int numOfRelevantStatements, int numOfConstantsOfQuery) {
		this.reasoningTime = reasoningTime;
		this.queryLength = queryLength;
		this.numOfRelevantStatements = numOfRelevantStatements;
		this.numOfConstantsOfQuery = numOfConstantsOfQuery;
	}

	public Measurement(double reasoningTime, int queryLength, int numOfRelevantStatements, int numOfConstantsOfQuery, double hashmapLookupsTime, double tcApplicationTime) {
		this.reasoningTime = (double) reasoningTime/1000000;
		this.queryLength = queryLength;
		this.numOfRelevantStatements = numOfRelevantStatements;
		this.numOfConstantsOfQuery = numOfConstantsOfQuery;
		this.hashmapLookupsTime = (double) hashmapLookupsTime/1000000;
		this.tcApplicationTime = (double) tcApplicationTime/1000000;
	}
	
	public static void groupByLength(List<Measurement> measurements) {
		
		HashMap<Integer, List<Double>> hashmap = new HashMap<Integer, List<Double>>();
		for(Measurement measurement : measurements) {
			int groupParameter = measurement.queryLength;
			if(!hashmap.containsKey(groupParameter)) {
				List<Double> thisList = new ArrayList<Double>();
				thisList.add(measurement.reasoningTime);
				hashmap.put(groupParameter, thisList);
			} else {
				hashmap.get(groupParameter).add(measurement.reasoningTime);
			}
		}
		
		// pretty printing
		System.out.println("####################");
		System.out.println("STATISTICS");
		System.out.println("Grouped by query length:");
		System.out.println("query length, number of queries, average completeness reasoning time");
		List<Integer> integerList = new ArrayList<Integer>(hashmap.keySet());
		Collections.sort(integerList);
		for(int key : integerList) {
			System.out.println(key + ", " + hashmap.get(key).size() + ", " + avgOf(hashmap.get(key)));
		}
		
	}
	
	public static void giveDetailedReasoningTimeGroupByLength(List<Measurement> measurements) {
		
		HashMap<Integer, List<Double>> hashmap = new HashMap<Integer, List<Double>>();
		for(Measurement measurement : measurements) {
			int groupParameter = measurement.queryLength;
			if(!hashmap.containsKey(groupParameter)) {
				List<Double> thisList = new ArrayList<Double>();
				thisList.add(measurement.reasoningTime);
				hashmap.put(groupParameter, thisList);
			} else {
				hashmap.get(groupParameter).add(measurement.reasoningTime);
			}
		}
		
		HashMap<Integer, List<Double>> hashmapLookups = new HashMap<Integer, List<Double>>();
		for(Measurement measurement : measurements) {
			int groupParameter = measurement.queryLength;
			if(!hashmapLookups.containsKey(groupParameter)) {
				List<Double> thisList = new ArrayList<Double>();
				thisList.add(measurement.hashmapLookupsTime);
				hashmapLookups.put(groupParameter, thisList);
			} else {
				hashmapLookups.get(groupParameter).add(measurement.hashmapLookupsTime);
			}
		}
		

		HashMap<Integer, List<Double>> hashmapTC = new HashMap<Integer, List<Double>>();
		for(Measurement measurement : measurements) {
			int groupParameter = measurement.queryLength;
			if(!hashmapTC.containsKey(groupParameter)) {
				List<Double> thisList = new ArrayList<Double>();
				thisList.add(measurement.tcApplicationTime);
				hashmapTC.put(groupParameter, thisList);
			} else {
				hashmapTC.get(groupParameter).add(measurement.tcApplicationTime);
			}
		}
		
		// pretty printing
		System.out.println("####################");
		System.out.println("STATISTICS");
		System.out.println("Grouped by query length:");
		System.out.println("query length, number of queries, average completeness reasoning time, average hashmap lookup time, average tc-application time");
		List<Integer> integerList = new ArrayList<Integer>(hashmap.keySet());
		Collections.sort(integerList);
		for(int key : integerList) {
			System.out.println(key + ", " + hashmap.get(key).size() + ", " + avgOf(hashmap.get(key)) + ", " + avgOf(hashmapLookups.get(key)) + ", " + avgOf(hashmapTC.get(key)));
		}
		
	}
	
	public static void groupByRelevantStatements(List<Measurement> measurements) {
		
		HashMap<Integer, List<Double>> hashmap = new HashMap<Integer, List<Double>>();
		for(Measurement measurement : measurements) {
			int groupParameter = measurement.numOfRelevantStatements;
			if(!hashmap.containsKey(groupParameter)) {
				List<Double> thisList = new ArrayList<Double>();
				thisList.add(measurement.reasoningTime);
				hashmap.put(groupParameter, thisList);
			} else {
				hashmap.get(groupParameter).add(measurement.reasoningTime);
			}
		}
		
		// pretty printing
		System.out.println("####################");
		System.out.println("STATISTICS");
		System.out.println("Grouped by number of relevant statements:");
		System.out.println("number of relevant statements, number of queries, average completeness reasoning time");
		List<Integer> integerList = new ArrayList<Integer>(hashmap.keySet());
		Collections.sort(integerList);
		for(int key : integerList) {
			System.out.println(key + ", " + hashmap.get(key).size() + ", " + avgOf(hashmap.get(key)));
		}
		
	}

	public static void giveDetailedReasoningTimeGroupByRelevantStatements(List<Measurement> measurements) {
		
		HashMap<Integer, List<Double>> hashmap = new HashMap<Integer, List<Double>>();
		for(Measurement measurement : measurements) {
			int groupParameter = measurement.numOfRelevantStatements;
			if(!hashmap.containsKey(groupParameter)) {
				List<Double> thisList = new ArrayList<Double>();
				thisList.add(measurement.reasoningTime);
				hashmap.put(groupParameter, thisList);
			} else {
				hashmap.get(groupParameter).add(measurement.reasoningTime);
			}
		}
		
		HashMap<Integer, List<Double>> hashmapLookups = new HashMap<Integer, List<Double>>();
		for(Measurement measurement : measurements) {
			int groupParameter = measurement.numOfRelevantStatements;
			if(!hashmapLookups.containsKey(groupParameter)) {
				List<Double> thisList = new ArrayList<Double>();
				thisList.add(measurement.hashmapLookupsTime);
				hashmapLookups.put(groupParameter, thisList);
			} else {
				hashmapLookups.get(groupParameter).add(measurement.hashmapLookupsTime);
			}
		}
		

		HashMap<Integer, List<Double>> hashmapTC = new HashMap<Integer, List<Double>>();
		for(Measurement measurement : measurements) {
			int groupParameter = measurement.numOfRelevantStatements;
			if(!hashmapTC.containsKey(groupParameter)) {
				List<Double> thisList = new ArrayList<Double>();
				thisList.add(measurement.tcApplicationTime);
				hashmapTC.put(groupParameter, thisList);
			} else {
				hashmapTC.get(groupParameter).add(measurement.tcApplicationTime);
			}
		}
		
		// pretty printing
		System.out.println("####################");
		System.out.println("STATISTICS");
		System.out.println("Grouped by number of relevant statements:");
		System.out.println("number of relevant statements, number of queries, average completeness reasoning time, average hashmap lookup time, average tc-application time");
		List<Integer> integerList = new ArrayList<Integer>(hashmap.keySet());
		Collections.sort(integerList);
		for(int key : integerList) {
			System.out.println(key + ", " + hashmap.get(key).size() + ", " + avgOf(hashmap.get(key)) + ", " + avgOf(hashmapLookups.get(key)) + ", " + avgOf(hashmapTC.get(key)));
		}
		
	}
	
	public static void groupByNumOfConstants(List<Measurement> measurements) {

		HashMap<Integer, List<Double>> hashmap = new HashMap<Integer, List<Double>>();
		for(Measurement measurement : measurements) {
			int groupParameter = measurement.numOfConstantsOfQuery;
			if(!hashmap.containsKey(groupParameter)) {
				List<Double> thisList = new ArrayList<Double>();
				thisList.add(measurement.reasoningTime);
				hashmap.put(groupParameter, thisList);
			} else {
				hashmap.get(groupParameter).add(measurement.reasoningTime);
			}
		}
		
		// pretty printing
		System.out.println("####################");
		System.out.println("STATISTICS");
		System.out.println("Grouped by number of constants of queries:");
		System.out.println("number of constants of queries, number of queries, average completeness reasoning time");
		List<Integer> integerList = new ArrayList<Integer>(hashmap.keySet());
		Collections.sort(integerList);
		for(int key : integerList) {
			System.out.println(key + ", " + hashmap.get(key).size() + ", " + avgOf(hashmap.get(key)));
		}
		
	}
	
	public static void giveDetailedReasoningTimeGroupByNumOfConstants(List<Measurement> measurements) {
		
		HashMap<Integer, List<Double>> hashmap = new HashMap<Integer, List<Double>>();
		for(Measurement measurement : measurements) {
			int groupParameter = measurement.numOfConstantsOfQuery;
			if(!hashmap.containsKey(groupParameter)) {
				List<Double> thisList = new ArrayList<Double>();
				thisList.add(measurement.reasoningTime);
				hashmap.put(groupParameter, thisList);
			} else {
				hashmap.get(groupParameter).add(measurement.reasoningTime);
			}
		}
		
		HashMap<Integer, List<Double>> hashmapLookups = new HashMap<Integer, List<Double>>();
		for(Measurement measurement : measurements) {
			int groupParameter = measurement.numOfConstantsOfQuery;
			if(!hashmapLookups.containsKey(groupParameter)) {
				List<Double> thisList = new ArrayList<Double>();
				thisList.add(measurement.hashmapLookupsTime);
				hashmapLookups.put(groupParameter, thisList);
			} else {
				hashmapLookups.get(groupParameter).add(measurement.hashmapLookupsTime);
			}
		}
		

		HashMap<Integer, List<Double>> hashmapTC = new HashMap<Integer, List<Double>>();
		for(Measurement measurement : measurements) {
			int groupParameter = measurement.numOfConstantsOfQuery;
			if(!hashmapTC.containsKey(groupParameter)) {
				List<Double> thisList = new ArrayList<Double>();
				thisList.add(measurement.tcApplicationTime);
				hashmapTC.put(groupParameter, thisList);
			} else {
				hashmapTC.get(groupParameter).add(measurement.tcApplicationTime);
			}
		}
		
		// pretty printing
		System.out.println("####################");
		System.out.println("STATISTICS");
		System.out.println("Grouped by number of constants of queries:");
		System.out.println("number of constants of queries, number of queries, average completeness reasoning time, average hashmap lookup time, average tc-application time");
		List<Integer> integerList = new ArrayList<Integer>(hashmap.keySet());
		Collections.sort(integerList);
		for(int key : integerList) {
			System.out.println(key + ", " + hashmap.get(key).size() + ", " + avgOf(hashmap.get(key)) + ", " + avgOf(hashmapLookups.get(key)) + ", " + avgOf(hashmapTC.get(key)));
		}
		
	}
	
	public static void giveNumOfConstantsGroupByQueryLength(List<Measurement> measurements) {

		HashMap<Integer, List<Double>> hashmap = new HashMap<Integer, List<Double>>();
		for(Measurement measurement : measurements) {
			int groupParameter = measurement.queryLength;
			if(!hashmap.containsKey(groupParameter)) {
				List<Double> thisList = new ArrayList<Double>();
				thisList.add(new Double(measurement.numOfConstantsOfQuery));
				hashmap.put(groupParameter, thisList);
			} else {
				hashmap.get(groupParameter).add(new Double(measurement.numOfConstantsOfQuery));
			}
		}
		
		// pretty printing
		System.out.println("####################");
		System.out.println("STATISTICS");
		System.out.println("giveNumOfConstantsGroupByQueryLength:");
		System.out.println("query length, number of queries, average number of constants of queries");
		List<Integer> integerList = new ArrayList<Integer>(hashmap.keySet());
		Collections.sort(integerList);
		for(int key : integerList)
			System.out.println(key + ", " + hashmap.get(key).size() + ", " + avgOf(hashmap.get(key)));
		
		// for latex
		System.out.println(">> FOR LATEX");
		for(int key : integerList)
			System.out.println("(" + key + "," + avgOf(hashmap.get(key)) + ")");
	}
	
	public static void giveRelevantStatementsGroupByQueryLength(List<Measurement> measurements) {

		HashMap<Integer, List<Double>> hashmap = new HashMap<Integer, List<Double>>();
		for(Measurement measurement : measurements) {
			int groupParameter = measurement.queryLength;
			if(!hashmap.containsKey(groupParameter)) {
				List<Double> thisList = new ArrayList<Double>();
				thisList.add(new Double(measurement.numOfRelevantStatements));
				hashmap.put(groupParameter, thisList);
			} else {
				hashmap.get(groupParameter).add(new Double(measurement.numOfRelevantStatements));
			}
		}
		
		// pretty printing
		System.out.println("####################");
		System.out.println("STATISTICS");
		System.out.println("giveRelevantStatementsGroupByQueryLength:");
		System.out.println("query length, number of queries, average number of relevant statements");
		List<Integer> integerList = new ArrayList<Integer>(hashmap.keySet());
		Collections.sort(integerList);
		for(int key : integerList)
			System.out.println(key + ", " + hashmap.get(key).size() + ", " + avgOf(hashmap.get(key)));
		
		// for latex
		System.out.println(">> FOR LATEX");
		for(int key : integerList)
			System.out.println("(" + key + "," + avgOf(hashmap.get(key)) + ")");
	}
	
	public static void giveRelevantStatementsGroupByNumOfConstants(List<Measurement> measurements) {

		HashMap<Integer, List<Double>> hashmap = new HashMap<Integer, List<Double>>();
		for(Measurement measurement : measurements) {
			int groupParameter = measurement.numOfConstantsOfQuery;
			if(!hashmap.containsKey(groupParameter)) {
				List<Double> thisList = new ArrayList<Double>();
				thisList.add(new Double(measurement.numOfRelevantStatements));
				hashmap.put(groupParameter, thisList);
			} else {
				hashmap.get(groupParameter).add(new Double(measurement.numOfRelevantStatements));
			}
		}
		
		// pretty printing
		System.out.println("####################");
		System.out.println("STATISTICS");
		System.out.println("giveRelevantStatementsGroupByNumOfConstants:");
		System.out.println("number of constants of queries, number of queries, average number of relevant statements");
		List<Integer> integerList = new ArrayList<Integer>(hashmap.keySet());
		Collections.sort(integerList);
		for(int key : integerList)
			System.out.println(key + ", " + hashmap.get(key).size() + ", " + avgOf(hashmap.get(key)));
		
		// for latex
		System.out.println(">> FOR LATEX");
		for(int key : integerList)
			System.out.println("(" + key + "," + avgOf(hashmap.get(key)) + ")");
	}
	
	public static double avgOf(List<Double> list) {	
		Double total = new Double(0);
		for(Double el : list) {
			total = new Double(total + el);
		}
		return ((double) total)/list.size();
	}
	
	public String toString() {
		return "Reasoning time (in ms): " + reasoningTime + ", Query length: " + queryLength + ", Relevant statements: " + numOfRelevantStatements + ", Number of constants of query: " + numOfConstantsOfQuery;
	}
	
}
