package it.unibz.inf.semweb.ontology;

import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;

import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.ModelFactory;
import org.apache.jena.rdf.model.ResourceFactory;
import org.apache.jena.rdf.model.Statement;
import org.apache.jena.rdf.model.StmtIterator;
import org.apache.jena.util.FileManager;

public class ExtractRDFSFromOWL {

	public static void main(String[] args) {

		String dirName = "files\\rdfs-schema\\";
		String inputFileName = dirName + "2014-09-09-ontology.sorted_lgd.nt"; // swc_2009-05-09.rdf or dbpedia_2014.owl or 2014-09-09-ontology.sorted_lgd.nt
		
		// create an empty model
        Model model = ModelFactory.createDefaultModel();
        Model extractedModel = ModelFactory.createDefaultModel();

        InputStream in = FileManager.get().open( inputFileName );
        if (in == null) {
            throw new IllegalArgumentException( "File: " + inputFileName + " not found");
        }
        
        // read the RDF/XML file
//      model.read(in, "");
        model.read(in, "", "N-TRIPLE");

        StmtIterator stmtIt = model.listStatements();
		while(stmtIt.hasNext()) {
			Statement stmt = stmtIt.next();
			String subject = stmt.getSubject().toString();
			String predicate = stmt.getPredicate().toString();
			String object = stmt.getObject().toString();
			
			if(predicate.compareTo("http://www.w3.org/2000/01/rdf-schema#subClassOf")==0)
			{
				extractedModel.add(stmt);
			}
			else if(predicate.compareTo("http://www.w3.org/2000/01/rdf-schema#subPropertyOf")==0)
			{
				extractedModel.add(stmt);
			}
			else if(predicate.compareTo("http://www.w3.org/2000/01/rdf-schema#domain")==0)
			{
				extractedModel.add(stmt);
			}
			else if(predicate.compareTo("http://www.w3.org/2000/01/rdf-schema#range")==0)
			{
				extractedModel.add(stmt);
			}
			else if(predicate.compareTo("http://www.w3.org/2002/07/owl#equivalentClass")==0) {
				System.out.println(stmt);
				Statement statement1 = extractedModel.createStatement(ResourceFactory.createResource(subject), ResourceFactory.createProperty("http://www.w3.org/2000/01/rdf-schema#subClassOf"), ResourceFactory.createResource(object));
				Statement statement2 = extractedModel.createStatement(ResourceFactory.createResource(object), ResourceFactory.createProperty("http://www.w3.org/2000/01/rdf-schema#subClassOf"), ResourceFactory.createResource(subject));
				extractedModel.add(statement1);
				extractedModel.add(statement2);
			}
			else if(predicate.compareTo("http://www.w3.org/2002/07/owl#equivalentProperty")==0) {
				System.out.println(stmt);
				Statement statement1 = extractedModel.createStatement(ResourceFactory.createResource(subject), ResourceFactory.createProperty("http://www.w3.org/2000/01/rdf-schema#subPropertyOf"), ResourceFactory.createResource(object));
				Statement statement2 = extractedModel.createStatement(ResourceFactory.createResource(object), ResourceFactory.createProperty("http://www.w3.org/2000/01/rdf-schema#subPropertyOf"), ResourceFactory.createResource(subject));
				extractedModel.add(statement1);
				extractedModel.add(statement2);
			}
						
		}

		try {
		String fileName = inputFileName + "_rdfs.nt";
		FileWriter out = new FileWriter( fileName );
		    extractedModel.write( out, "NT" );
		}
	   catch (IOException closeException) {
	       closeException.printStackTrace();
	   }

	}

}
