package it.unibz.inf.semweb.statistics;

import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.google.common.collect.HashMultiset;
import com.google.common.collect.Multiset;
import org.apache.jena.query.QueryFactory;

import it.unibz.inf.semweb.QueryToolkit;

/**
 * @author Fariz Darari
 *
 * Compute the statistics of a query class in the logs
 * 
 */
public class QueryStatistics {

	public static final int QUERY_LIMIT = 5000;
	public static final String QUERY_ENDPOINT = "http://localhost:8890/sparql";

	public static void main(String[] args) {

		long individualQueryCounter = 0;
		long numberOfOperators = 0;
		boolean getMoreQueries = true;
		int getQueriesCounter = 0;
		Multiset<Integer> statistics = HashMultiset.create();

		while(getMoreQueries) {

			System.out.println("> Get queries iteration: " + getQueriesCounter);
			List<String> queryStrings = getQueries(getQueriesCounter);
			System.out.println("Got " + queryStrings.size() + " queries.");

			for(String queryString : queryStrings) {	

				individualQueryCounter++;
				String lowercasedQueryString = queryString.toLowerCase();
				int occurrences = StringUtils.countMatches(lowercasedQueryString, "optional");
				
				// count here
				statistics.add(new Integer(occurrences));

				numberOfOperators = numberOfOperators + occurrences;

			}

			getQueriesCounter++;
			if(queryStrings.isEmpty()) 
				getMoreQueries = false;
			
			System.out.println("---\nTotal queries so far: " + individualQueryCounter);
			
		}
		
		System.out.println("---\nTotal queries: " + individualQueryCounter);
		System.out.println("Average number of operators: " + ((double)numberOfOperators/(double)individualQueryCounter));
		
		//display the distinct elements of the multiset with their occurrence count
	    System.out.println("MultiSet [");
	     
        for (Multiset.Entry<Integer> entry : statistics.entrySet())
        {
          System.out.println("Element: " + entry.getElement() + ", Occurrence(s): " + entry.getCount());		    
        }
        System.out.println("]");
		
	}

	public static List<String> getQueries(int getQueriesCounter) {

		String getQueriesQueryStr = 
				"PREFIX lsqv: <http://lsq.aksw.org/vocab#> " +
				"SELECT ?queryStr WHERE { " +
				   "?id lsqv:endpoint <http://dbpedia.org/sparql> ; a <http://spinrdf.org/sp#Select> ; <http://spinrdf.org/sp#text> ?queryStr . " +
				   "FILTER EXISTS {?id lsqv:usesFeature lsqv:Optional} " +
				"} LIMIT " + QUERY_LIMIT + " OFFSET " + (QUERY_LIMIT * getQueriesCounter);
		
		return QueryToolkit.evalSELECT(QueryFactory.create(getQueriesQueryStr), QUERY_ENDPOINT);
		
	}

}
